/* -*- C -*- */
/* Time-stamp: "2006-02-27 22:37:07 fujiwara" */

/**********************************************************************/
/*  Additive Reduction for sine, cosine, exponential Function         */
/*                                                                    */
/*  Copyright (C) 2002, FUJIWARA Hiroshi                              */
/*        <fujiwara@acs.i.kyoto-u.ac.jp>                              */
/**********************************************************************/

static char rcsid[] = "$Id:$";

#include "exfloat.h"
#include "func.h"

/**********************************************************************/
/* Split x into integral and fractional parts                         */
/* each array argument has (1 + pr64) elements.                       */
/* return value : ip%8                                                */
/**********************************************************************/

long exflib_modf(exflib_cuQword x[], exflib_uQword ip[], exflib_Qword *k, long pr64)
{
    exflib_uQword tmp;
    long ret, i;
    exflib_Qword e = (x[0] & ~EXFLIB_MASK_SGN) - EXFLIB_BIAS;

    // e < 0, i.e. |x| < 1.
    if(e < 0){
	for(i = 0; i <= pr64 ; i++)
	    ip[i] = 0;
	*k = 0;
	return 0;
    }

    tmp = x[0];
    // e >= 0, i.e. |x| >= 1.
    ip[0] = 1; 	// hidden one
    for(i = 1; e-64 >= 0; e -= 64, i++)
	ip[i] = x[i];

    if(e){
	*k = ~(0ULL) << (64 - e);
	ip[i] = x[i] & (*k);

	*k = (ip[i-1] << e) | (ip[i] >> (64-e));
	ret = 0xffff & (*k);
	i++;

    } else {
	ret = ip[i-1] & 0x7ULL;
    }
    
    while(i <= pr64)
	ip[i++] = 0;

    ip[0] = tmp;

    return ret;
}

/**********************************************************************/
/* Decompose x = c * kk + r, (0 <= r < c), and return kk (mod 4)      */
/*                                                                    */
/* cc == 2 : x = kk * ln(2) + r,   0 <= r  < ln(2).                   */
/* cc == 3 : x = kk * pi/2 + r,    -pi/4 <= r <= pi/4.                */
/*                                                                    */
/* Assumption : x > 0                                                 */
/* array size : x[1+pr64], r[1 + pr64 + 1]                            */
/**********************************************************************/

long exflib_add_reduction(exflib_cuQword x[], long cc, exflib_uQword r[], exflib_ulong pr64, exflib_Qword *kkk)
{
    exflib_uQword xx[1+EXFLIB_CONST_MAX_PR], kk[1+EXFLIB_CONST_MAX_PR], e;
    long add_rdc_pr, k, i;

    static exflib_uQword exflib_ln2[] = {
#include "ln2.exfloat"
    };
    static exflib_uQword piq[] = {
#include "pi.exfloat"
    };

    exflib_uQword *c = (cc == 2)? exflib_ln2: piq;

    piq[0] = EXFLIB_BIAS - 1;	// piq = pi/4
                                // First, decomposing x = r + kk * (pi/4)

    // x <= 1/2 (x = 2^e * 1.F <= 1/2, i.e., 2^e <= 1/4)
    //  or
    // x < c ?
    if((x[0] <= EXFLIB_BIAS - 2) || exflib_float_lt(x, c, pr64)){

	for(i = 0; i <= pr64; i++)
	    r[i] = x[i];
	r[1+pr64] = 0;
	*kkk = 0;
	return 0;
    }

    // c <= x < 1
    if(x[0] == EXFLIB_BIAS - 1){

	add_rdc_pr = (1 + 2*pr64 < EXFLIB_CONST_MAX_PR)? 1 + 2*pr64: EXFLIB_CONST_MAX_PR;

	for(i = 0; i <= pr64; i++)
	    xx[i] = x[i];
	for(i = pr64+1; i <= add_rdc_pr; i++)
	    xx[i] = 0;

	if(cc == 3)
	    c[0] = EXFLIB_BIAS;	// c = pi/2

	exflib_float_sub(xx, xx, c, add_rdc_pr);

	for(i = 0; i <= pr64+1; i++)
	    r[i] = xx[i];

	*kkk = 1;
	return 1;
    }

    e = x[0] - EXFLIB_BIAS;	// e >= 0

    // temporary precision for guard
    add_rdc_pr = 1 + 2*pr64;
    if(e >= pr64*64 + 1){

	add_rdc_pr = pr64 + (e+1)/64;
	if((e+1)%64 != 0)
	    add_rdc_pr++;
    }

    if(add_rdc_pr > EXFLIB_CONST_MAX_PR)
	add_rdc_pr = EXFLIB_CONST_MAX_PR;

    // zero extend of argument
    for(i = 0; i <= pr64; i++)
	xx[i] = x[i];
    for(i = pr64+1; i <= add_rdc_pr; i++)
	xx[i] = 0;

    exflib_float_div(kk, xx, c, add_rdc_pr);  // kk >= 1

    if(e <= 29){

	long ee = kk[0] - EXFLIB_BIAS;
	k = 1;	// hidden one
	if(ee)
	    k = (k << ee) | (kk[1] >> (64 - ee));
	*kkk = k;
	if((cc == 3) && (k%2 == 1))
	    k++;

	exflib_float_imul(kk, c, k, add_rdc_pr);

    } else {

	// integer part of kk
	k = exflib_modf(kk, kk, kkk, add_rdc_pr);
	exflib_float_mul(kk, c, kk, add_rdc_pr);

	if((cc == 3) && (k%2 == 1)){
	    k++;
	    exflib_float_add(kk, kk, c, add_rdc_pr);
	}
    }

    // residual
    exflib_float_sub(xx, xx, kk, add_rdc_pr);
    for(i = 0; i <= pr64 + 1; i++)
	r[i] = xx[i];

    if(cc == 3)
	k /= 2;

    return k%4;
}

/* End of add_reduction.c */
