/* -*- C -*- */
/* Time-stamp: "2006-02-27 22:38:19 fujiwara" */

/**********************************************************************/
/*  Copyright (C) 2005, FUJIWARA Hiroshi                              */
/*        <fujiwara@acs.i.kyoto-u.ac.jp>                              */
/**********************************************************************/

static char rcsid[] = "$Id: $";

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "exfloat.h"
#include "func.h"

/**********************************************************************/
/*  v = ln2 * (e - EXFLIB_BIAS)                                          */
/**********************************************************************/

void exflib_load_ln2(exflib_uQword e, exflib_uQword v[], long pr64)
{
    exflib_uQword ln2[] = {
#include "ln2.fixed"
    };
    exflib_Qword se = (e & ~EXFLIB_SIGN) - EXFLIB_BIAS;
    long sign = 1;
    long i;

    if(se == 0){
	for(i = 0; i <= pr64; i++)
	    v[0] = 0;
	return;
    }

    if(se < 0){
	se = -se;
	sign = -1;
    }

    exflib_array_imul(v, ln2, se, pr64 + 1);
    if(v[0] == 0){
	exflib_sll_onebit(v, pr64 + 1);
	v[0] = EXFLIB_BIAS -1;
    } else if(v[0] == 1){
	v[0] = EXFLIB_BIAS;
    } else {
	exflib_array_to_exfloat(v, pr64 +1);
    }

    if(sign == -1)
	v[0] |= EXFLIB_SIGN;

    return;
}

/**********************************************************************/
/*  if(sign == 1)                                                     */
/*    x = 1/(2*n-1) + x, n : integer (>= 1)                           */
/*  else                                                              */
/*    x = 1/(2*n-1) - x, n : integer (>= 1)                           */
/*  The argument x[] has size elements.                               */
/**********************************************************************/

void exflib_invadd(exflib_uQword x[], long n, exflib_uQword table[], long sign, long size)
{
    exflib_uQword d[1+EXFLIB_CONST_MAX_PR];
    long i;

    /* initialize */
    if(table[0] == 0){
	table[0] = 1;	/* The first row (table[0]) holds '1' */
	for(i = 1; i < size; i++)
	    table[i] = 0;
    }

    if(n < EXFLIB_FUNCLOG_INVTBL_MAXN){

	if((n > 1) && (*(table + (n-1)*size + 1) == 0)){
	    i = 0;
	    exflib_array_idiv(table + (n-1)*size, table, 2*n-1, size, &i);
	}

	/* read from table */
	if(sign == 1)
	    exflib_array_add(x, x, table + (n-1)*size, size);
	else
	    exflib_array_sub(x, table + (n-1)*size, x, size);


    } else {

	i = 0;
	exflib_array_idiv(d, table, 2*n-1, size, &i);
	
	if(sign == 1)
	    exflib_array_add(x, x, d, size);
	else
	    exflib_array_sub(x, d, x, size);
    }

    return;
}

/**********************************************************************/
/*   v = 2(y + y^3/3 + y^5/5 + ...)                                   */
/* Assumption : 0 <= y <= 1/3 (y = 0.F/2.F)                           */
/**********************************************************************/

void exflib_array_log(exflib_uQword y[], exflib_uQword v[], exflib_uQword table[], long pr64)
{
    exflib_uQword xx[1+EXFLIB_CONST_MAX_PR];
    exflib_Qword ee;
    long e, i, n;

    ee = (y[0] & ~EXFLIB_SIGN) - EXFLIB_BIAS;
    if(ee <= 1.5*log(3.0)/log(2.0) - 1.5 - 32*pr64){
	// return 2*y
	v[0] = y[0] + 1;
	for(i = 1; i <= pr64; i++)
	    v[i] = y[i];
	return;
    }

    e = (long)ee;
    n = exflib_taylor_tranc_log(pr64, e);

    exflib_exfloat_to_array(y, e, v, pr64+1);
    exflib_array_mul(xx, v, v, pr64+1);

    for(i = 0; i <= pr64; i++)
	v[0] = 0;
    exflib_invadd(v, n+1, table, 1, pr64+1);

    while(n > 0){
	exflib_array_mul(v, v, xx, pr64+1);
	exflib_invadd(v, n, table, 1, pr64+1);
	n--;
    }

    v[0] = EXFLIB_BIAS + 1;
    exflib_float_mul(v, v, y, pr64);

    return;
}

/***********************************************************************/
/*  yy[]   : ln(x),  	(1+pr64) elements                              */
/*  x[]   : input,    	(1+pr64) elements                              */
/*                                                                     */
/*  w[] : work area, 	(1+pr64+guard)*3 elements                      */
/***********************************************************************/

exflib_ulong exflib_float_log(exflib_uQword yy[], exflib_cuQword x[], const long pr64, exflib_uQword w[], exflib_uQword *inv_table)
{
    const long guard = 1;		/* guard digits */
    const long pr64g = pr64 + guard;	/* extended precision */

    exflib_uQword *eln2 = w;
    exflib_uQword *y = w + (1+pr64g);
    exflib_uQword *one = w + (1+pr64g)*2;
    exflib_uQword mask = 0;
    exflib_ulong ret;

    long i;

    /* domain check */
    if((x[0] & EXFLIB_MASK_SGN) || (x[0] == 0) || (x[0] == EXFLIB_MASK_SGN)){
	fprintf(stderr, "log : domain error\n");
	exit(1);
    }

    /* e x log(2) */
    exflib_load_ln2(x[0], eln2, pr64g);

    /* check mantissa of x == 1 */
    for(i = 1; (i <= pr64) & !mask; i++)
	mask |= x[i];
    if(!mask){
	for(i = 0; i <= pr64; i++)
	    y[i] = eln2[i];

	if(sizeof(long) == 8)		/* 64-bit architecture */
	    ret = eln2[pr64+1];
	else if(sizeof(long) == 4)	/* 32-bit architecture */
	    ret = eln2[pr64+1] >> 32;
	
	for(i = 0; i <= pr64; i++)
	    yy[i] = eln2[i];

	return ret;
    }

    /* y = (1.F - 1)/(1.F + 1) = 1 - 2/(1.F+1) */

    /* zero extension of x */
    y[0] = EXFLIB_BIAS;
    y[1] = x[1] >> 1;
    for(i = 2; i <= pr64; i++)
	y[i] = (x[i-1] << 63) | (x[i] >> 1);
    y[pr64+1] = x[pr64] << 63;
    for(i++; i <= pr64g; i++)
	y[i] = 0;
    
    /* one = 1 */
    one[0] = EXFLIB_BIAS;
    for(i = 1; i <= pr64g; i++)
	one[i] = 0;

    exflib_float_div(y, one, y, pr64g);
    exflib_float_sub(y, one, y, pr64g);
    exflib_array_log(y, one, inv_table, pr64g);
    exflib_float_add(y, one, eln2, pr64g);

    for(i = 0; i <= pr64; i++)
	yy[i] = y[i];
    
    if(sizeof(long) == 8)		/* 64-bit architecture */
	ret = y[pr64+1];
    else if(sizeof(long) == 4)	/* 32-bit architecture */
	ret = y[pr64+1] >> 32;

    return ret;
}

/* End of func_log.c */
