// -*- C++ -*-
// $Id: exfutil.h,v 1.2 2002/04/10 08:31:11 fujiwara Exp $
#ifndef EXFUTIL_H
#define EXFUTIL_H

static const char EXFUTIL_H_VERSION[] = "Time-stamp: \"2006-06-15 10:17:24 fujiwara\" ";

//----------------------------------------------------------------------
//  Utility Routines for extend float (exfloat)
//
//  Copyright (C) 2002-2005, FUJIWARA Hiroshi
//        <fujiwara@acs.i.kyoto-u.ac.jp>
//----------------------------------------------------------------------
#include <iostream>

//----------------------------------------------------------------------
//  Factorial
//----------------------------------------------------------------------

exfloat fact(const exfloat& x)
{
    exfloat f = 1;
    for(int i = 2; i <= x; i++)
	f *= i;
    return f;
}

//----------------------------------------------------------------------
//  Bessel Functions
//----------------------------------------------------------------------

// Bessel Function of the First Kind, J_n(x)

exfloat bessel_J(const long n, const exfloat& x)
{
    const long guard = 2; // guard digit
    const long pr64g = exfloat::precision64 + guard;
    const long sizeg = exfloat::size + guard;

    exflib_uQword bes[sizeg], xx[sizeg], term[sizeg], tmp[sizeg], y[sizeg];
    long i;

    // domain check
    if(x.num[0] & (~EXFLIB_MASK_SGN) == 0){

	exflib::work.num[0] = (n == 0)? EXFLIB_BIAS: 0;
	for(long i = 1; i < exfloat::size; i++)
	    exflib::work.num[i] = 0;

	return exflib::work;
    }
    if(x.num[0] & EXFLIB_MASK_SGN){
	std::cerr << "bessel_J : domain error\n";
	exit(1);
    }

    // copy and zero padding
    if(x.num[0] & ~EXFLIB_MASK_SGN)
	xx[0] = x.num[0] - 1;
    else
	xx[0] = 0;
    for(i = 1; i < exfloat::size; i++)
	xx[i] = x.num[i];
    for(; i < sizeg; i++)
	xx[i] = 0;

    // initialize
    term[0] = EXFLIB_BIAS;
    for(i = 1; i < sizeg; i++)
	term[i] = 0;
    for(i = 2; i <= n; i++)
	exflib_float_idiv(term, term, i, pr64g);
    for(i = 0; i < sizeg; i++)
	bes[i] = term[i];

    exflib_float_mul(y, xx, xx, pr64g);

    // expansion
    for(long k = 1; ; k++){
	exflib_float_mul(term, term, y, pr64g);
	exflib_float_idiv(term, term, k*(k+n), pr64g);
	term[0] += EXFLIB_MASK_SGN;
	exflib_float_add(tmp, bes, term, pr64g);

	// end condition
	i = 0;
	while((i < sizeg) && (tmp[i] == bes[i]))
	    i++;
	if(i == sizeg)
	    break;

	// renewal
	for(i = 0; i < sizeg; i++)
	    bes[i] = tmp[i];
    }

    for(i = n; i > 0; i >>= 1){

	if(i % 2)
	    exflib_float_mul(bes, bes, xx, pr64g);
	exflib_float_mul(xx, xx, xx, pr64g);
    }

    for(i = 0; i < exfloat::size; i++)
	exflib::work.num[i] = bes[i];

    return exflib::work;
}

// Zeroth order Bessel Functions of the Second Kind
// (Neumann Function, N_0(x), Y_0(x))

exfloat bessel_Y0(const exfloat &x)
{
    static exfloat GAMMA = "#g";
    static exfloat PI = "#pi";

    const long guard = 2;
    const long pr64g = exfloat::precision64 + guard;
    const long sizeg = exfloat::size + guard;

    exflib_uQword bes[sizeg], harmonic[sizeg], term[sizeg],
	one[sizeg], tmp[sizeg], x2[sizeg];
    long i;

    // domain check
    if((x.num[0] & EXFLIB_MASK_SGN) || (x.num[0] == 0)){
	std::cerr << "bessel_Y0 : domain error\n";
	exit(1);
    }

    // initialize
    for(i = 0; i < sizeg; i++)
	bes[i] = harmonic[i] = one[i] = term[i] = 0;
    one[0] = 0x3fffffffffffffffULL;
    term[0] = 0x3fffffffffffffffULL;

    // making -(z/2)^2 = -(z^2)/4
    for(i = 0; i < exfloat::size; i++)
	tmp[i] = x.num[i];
    for(; i < sizeg; i++)
	tmp[i] = 0;
    exflib_float_mul(x2, tmp, tmp, pr64g);
    x2[0] -= 2;
    x2[0] += 0x8000000000000000ULL;

    // sum
    for(long k = 1; ; k++){

	// harmonic += 1/k
	exflib_float_idiv(tmp, one, k, pr64g);
	exflib_float_add(harmonic, harmonic, tmp, pr64g);

	// term /= k*k
	exflib_float_idiv(term, term, k*k, pr64g);
	// term *= x*x/4
	exflib_float_mul(term, term, x2, pr64g);

	exflib_float_mul(tmp, term, harmonic, pr64g);

	exflib_float_add(tmp, bes, tmp, pr64g);

	// end condition
	int j = 0;
	while((j < sizeg) && (tmp[j] == bes[j]))
	    j++;
	if(j == sizeg)
	    break;

	// renewal
	for(j = 0; j < sizeg; j++)
	    bes[j] = tmp[j];
    }

    exfloat term1 = bessel_J(0, x) * (GAMMA + log(x/2));

    exflib_float_sub(bes, term1.num, bes, exfloat::precision64);
    exflib_float_div(bes, bes, PI.num, exfloat::precision64);
    bes[0] += 1;

    for(i = 0; i < exfloat::size; i++)
	exflib::work.num[i] = bes[i];

    return exflib::work;
}

// First order Bessel Functions of the Second Kind
// (Neumann Function, N_1(x), Y_1(x))

exfloat bessel_Y1(const exfloat &x)
{
    static exfloat GAMMA = "#g";
    static exfloat PI = "#pi";

    const long guard = 2;
    const long pr64g = exfloat::precision64 + guard;
    const long sizeg = exfloat::size + guard;

    exflib_uQword bes[sizeg], harmonic[sizeg], inv[sizeg],
	term[sizeg], one[sizeg], tmp[sizeg], x2[sizeg];
    long i;

    // domain check
    if((x.num[0] & EXFLIB_MASK_SGN) || (x.num[0] == 0)){
	std::cerr << "bessel_Y1 : domain error\n";
	exit(1);
    }

    // initialize
    for(i = 0; i < sizeg; i++)
	harmonic[i] = inv[i] = one[i] = 0;
    inv[0] = one[0] = 0x3fffffffffffffffULL;

    // zero extension of x
    for(i = 0; i < exfloat::size; i++)
	bes[i] = term[i] = x.num[i];
    for(; i < sizeg; i++)
	bes[i] = term[i] = 0;

    // making -(z/2)^2 = -(z^2)/4
    exflib_float_mul(x2, term, term, pr64g);
    x2[0] -= 2;
    x2[0] += 0x8000000000000000ULL;

    // term = x/4
    term[0] -= 2;
    // bes = x/4 (as k == 0)
    bes[0] -= 2;

    // sum
    for(long k = 1; ; k++){

	// term /= k*(k+1)
	exflib_float_idiv(term, term, k*(k+1), pr64g);
	// term *= (-x*x/4)
	exflib_float_mul(term, term, x2, pr64g);

	// harmonic += 2/k
	// harmonic  = 2*(1+ 1/2 + 1/3 + ... + 1/k)
	inv[0] += 1;
	exflib_float_add(harmonic, harmonic, inv, pr64g);

	// inv = 1/(k+1)
	exflib_float_idiv(inv, one, k+1, pr64g);
	// tmp = 2*(1+ 1/2 + 1/3 + ... + 1/k) + 1/(k+1)
	exflib_float_add(tmp, harmonic, inv, pr64g);
	
	exflib_float_mul(tmp, term, tmp, pr64g);

	exflib_float_add(tmp, bes, tmp, pr64g);

	// end condition
	int j = 0;
	while((j < sizeg) && (tmp[j] == bes[j]))
	    j++;
	if(j == sizeg)
	    break;

	// renewal
	for(j = 0; j < sizeg; j++)
	    bes[j] = tmp[j];
    }

    exflib_float_div(exflib::work2.num, one, x.num, exfloat::precision64);
    exflib::work = bessel_J(1, x) * (GAMMA + log(x/2)) - exflib::work2;

    exflib_float_sub(bes, exflib::work.num, bes, exfloat::precision64);
    exflib_float_div(exflib::work.num, bes, PI.num, exfloat::precision64);
    exflib::work.num[0] += 1;

    return exflib::work;
}

// Modified Bessel Function of the First Kind, I_k(x)

exfloat bessel_I(const long n, const exfloat& x)
{
    const long guard = 2; // guard digit
    const long pr64g = exfloat::precision64 + guard;
    const long sizeg = exfloat::size + guard;

    exflib_uQword bes[sizeg], xx[sizeg], term[sizeg], tmp[sizeg], y[sizeg];
    long i;

    // domain check
    if(x.num[0] & (~EXFLIB_MASK_SGN) == 0){

	exflib::work.num[0] = (n == 0)? EXFLIB_BIAS: 0;
	for(long i = 1; i < exfloat::size; i++)
	    exflib::work.num[i] = 0;

	return exflib::work;
    }
    if(x.num[0] & EXFLIB_MASK_SGN){
	std::cerr << "bessel_I : domain error\n";
	exit(1);
    }

    // copy and zero padding
    if(x.num[0] & ~EXFLIB_MASK_SGN)
	xx[0] = x.num[0] - 1;
    else
	xx[0] = 0;
    for(i = 1; i < exfloat::size; i++)
	xx[i] = x.num[i];
    for(; i < sizeg; i++)
	xx[i] = 0;

    // initialize
    term[0] = EXFLIB_BIAS;
    for(i = 1; i < sizeg; i++)
	term[i] = 0;
    for(i = 2; i <= n; i++)
	exflib_float_idiv(term, term, i, pr64g);
    for(i = 0; i < sizeg; i++)
	bes[i] = term[i];

    exflib_float_mul(y, xx, xx, pr64g);

    // expansion
    for(long k = 1; ; k++){
	exflib_float_mul(term, term, y, pr64g);
	exflib_float_idiv(term, term, k*(k+n), pr64g);
	exflib_float_add(tmp, bes, term, pr64g);

	// end condition
	i = 0;
	while((i < sizeg) && (tmp[i] == bes[i]))
	    i++;
	if(i == sizeg)
	    break;

	// renewal
	for(i = 0; i < sizeg; i++)
	    bes[i] = tmp[i];
    }

    for(i = n; i > 0; i >>= 1){

	if(i % 2)
	    exflib_float_mul(bes, bes, xx, pr64g);
	exflib_float_mul(xx, xx, xx, pr64g);
    }

    for(i = 0; i < exfloat::size; i++)
	exflib::work.num[i] = bes[i];

    return exflib::work;
}

// Zeroth order Modified Bessel Functions of the Second Kind K_0(x)

exfloat bessel_K0(const exfloat &x)
{
    static exfloat GAMMA = "#g";
    static exfloat PI = "#pi";

    const long guard = 2;
    const long pr64g = exfloat::precision64 + guard;
    const long sizeg = exfloat::size + guard;

    exflib_uQword bes[sizeg], harmonic[sizeg], term[sizeg],
	one[sizeg], tmp[sizeg], x2[sizeg];
    long i;

    // domain check
    if((x.num[0] & EXFLIB_MASK_SGN) || (x.num[0] == 0)){
	std::cerr << "bessel_K0 : domain error\n";
	exit(1);
    }

    // initialize
    for(i = 0; i < sizeg; i++)
	bes[i] = harmonic[i] = one[i] = term[i] = 0;
    one[0] = 0x3fffffffffffffffULL;
    term[0] = 0x3fffffffffffffffULL;

    // making (z/2)^2 = z^2/4
    for(i = 0; i < exfloat::size; i++)
	tmp[i] = x.num[i];
    for(; i < sizeg; i++)
	tmp[i] = 0;
    exflib_float_mul(x2, tmp, tmp, pr64g);
    x2[0] -= 2;

    // sum
    for(long k = 1; ; k++){

	// harmonic += 1/k
	exflib_float_idiv(tmp, one, k, pr64g);
	exflib_float_add(harmonic, harmonic, tmp, pr64g);

	// term /= k*k
	exflib_float_idiv(term, term, k*k, pr64g);
	// term *= x*x/4
	exflib_float_mul(term, term, x2, pr64g);
	
	exflib_float_mul(tmp, term, harmonic, pr64g);

	exflib_float_add(tmp, bes, tmp, pr64g);

	// end condition
	int j = 0;
	while((j < sizeg) && (tmp[j] == bes[j]))
	    j++;
	if(j == sizeg)
	    break;

	// renewal
	for(j = 0; j < sizeg; j++)
	    bes[j] = tmp[j];
    }

    exfloat term1 = bessel_I(0, x) * (GAMMA + log(x/2));
    
    exflib_float_sub(bes, bes, term1.num, exfloat::precision64);
    
    for(i = 0; i < exfloat::size; i++)
	exflib::work.num[i] = bes[i];

    return exflib::work;
}

// First order Modified Bessel Functions of the Second Kind K_1(x)

exfloat bessel_K1(const exfloat &x)
{
    static exfloat GAMMA = "#g";
    static exfloat PI = "#pi";

    const long guard = 2;
    const long pr64g = exfloat::precision64 + guard;
    const long sizeg = exfloat::size + guard;

    exflib_uQword bes[sizeg], harmonic[sizeg], inv[sizeg],
	term[sizeg], one[sizeg], tmp[sizeg], x2[sizeg];
    long i;

    // domain check
    if((x.num[0] & EXFLIB_MASK_SGN) || (x.num[0] == 0)){
	std::cerr << "bessel_K1 : domain error\n";
	exit(1);
    }

    // initialize
    for(i = 0; i < sizeg; i++)
	harmonic[i] = inv[i] = one[i] = 0;
    inv[0] = one[0] = 0x3fffffffffffffffULL;

    // zero extension of x
    for(i = 0; i < exfloat::size; i++)
	bes[i] = term[i] = x.num[i];
    for(; i < sizeg; i++)
	bes[i] = term[i] = 0;

    // making (z/2)^2 = z^2/4
    exflib_float_mul(x2, term, term, pr64g);
    x2[0] -= 2;

    // term = x/4
    term[0] -= 2;
    // bes = x/4 (as k == 0)
    bes[0] -= 2;

    // sum
    for(long k = 1; ; k++){

	// term /= k*(k+1)
	exflib_float_idiv(term, term, k*(k+1), pr64g);
	// term *= x*x/4
	exflib_float_mul(term, term, x2, pr64g);
	
	// harmonic += 2/k
	// harmonic  = 2*(1+ 1/2 + 1/3 + ... + 1/k)
	inv[0] += 1;
	exflib_float_add(harmonic, harmonic, inv, pr64g);

	// inv = 1/(k+1)
	exflib_float_idiv(inv, one, k+1, pr64g);
	// tmp = 2*(1+ 1/2 + 1/3 + ... + 1/k) + 1/(k+1)
	exflib_float_add(tmp, harmonic, inv, pr64g);

	exflib_float_mul(tmp, term, tmp, pr64g);

	exflib_float_add(tmp, bes, tmp, pr64g);

	// end condition
	int j = 0;
	while((j < sizeg) && (tmp[j] == bes[j]))
	    j++;
	if(j == sizeg)
	    break;

	// renewal
	for(j = 0; j < sizeg; j++)
	    bes[j] = tmp[j];
    }
    
    exflib_float_div(exflib::work2.num, one, x.num, exfloat::precision64);
    exflib::work = bessel_I(1, x) * (GAMMA + log(x/2)) + exflib::work2;

    exflib_float_sub(exflib::work.num, exflib::work.num, bes, exfloat::precision64);
    return exflib::work;
}

// Spherical Bessel Function of the First Kind

exfloat bessel_j(const long n, const exfloat &x)
{
    const long guard = 1;	// guard digit
    const long pr64g = exfloat::precision64 + guard;
    const long sizeg = exfloat::size + guard;

    exflib_uQword bes[sizeg], xx[sizeg], term[sizeg], tmp[sizeg], y[sizeg];
    long i;

    if((x.num[0] == 0) || (x.num[0] == EXFLIB_MASK_SGN))
	if(n == 0)
	    return static_cast<exfloat>(1);
	else
	    return static_cast<exfloat>(0);

    // copy and zero padding
    for(i = 0; i < exfloat::size; i++)
	xx[i] = x.num[i];
    for(; i < sizeg; i++)
	xx[i] = 0;
    exflib_float_mul(y, xx, xx, pr64g);
    
    // initialize
    // term = 1/(1 \cdot 3 \cdot 5 \cdots (2n+1))
    term[0] = EXFLIB_BIAS;
    for(i = 1; i < sizeg; i++)
	term[i] = 0;
    for(i = 3; i <= 2*n+1; i += 2)
	exflib_float_idiv(term, term, i, pr64g);
    for(i = 0; i < sizeg; i++)
	bes[i] = term[i];

    // expansion
    for(long k = 1; ; k++){
	exflib_float_mul(term, term, y, pr64g);
	exflib_float_idiv(term, term, 2*k*(2*n+2*k+1), pr64g);
	term[0] += EXFLIB_MASK_SGN;
	exflib_float_add(tmp, bes, term, pr64g);

	// end condition
	i = 0;
	while((i < sizeg) && (tmp[i] == bes[i]))
	    i++;
	if(i == sizeg)
	    break;

	// renewal
	for(i = 0; i < sizeg; i++)
	    bes[i] = tmp[i];
    }

    // bes *= x^n
    for(i = n; i > 0; i >>= 1){

	if(i % 2)
	    exflib_float_mul(bes, bes, xx, pr64g);
	exflib_float_mul(xx, xx, xx, pr64g);
    }
    
    for(i = 0; i < exfloat::size; i++)
	exflib::work.num[i] = bes[i];

    return exflib::work;
}

// Spherical Bessel Function of the Second Kind (Spherical Nuemann Function)

exfloat bessel_y(const long n, const exfloat& x)
{
    const long guard = 1;	// guard digit
    const long pr64g = exfloat::precision64 + guard;
    const long sizeg = exfloat::size + guard;

    exflib_uQword bes[sizeg], xx[sizeg], term[sizeg], tmp[sizeg], y[sizeg];
    long i;

    if((x.num[0] == 0) || (x.num[0] == EXFLIB_MASK_SGN)){
	exflib::work.num[0] = 0xffffffffffffffffULL;
	return exflib::work;
    }
    
    // copy and zero padding
    for(i = 0; i < exfloat::size; i++)
	xx[i] = x.num[i];
    for(; i < sizeg; i++)
	xx[i] = 0;
    exflib_float_mul(y, xx, xx, pr64g);

    // initialize
    // term = 1
    term[0] = EXFLIB_BIAS;
    for(i = 1; i < sizeg; i++)
	term[i] = 0;
    for(i = 0; i < sizeg; i++)
	bes[i] = term[i];

    // expansion
    for(long k = 1; ; k++){
	exflib_float_mul(term, term, y, pr64g);

	if(k <= n)
	    exflib_float_idiv(term, term, 2*k*(2*n-2*k+1), pr64g);
	else
	    exflib_float_idiv(term, term, -2*k*(2*k-2*n-1), pr64g);

	exflib_float_add(tmp, bes, term, pr64g);

	// end condition
	i = 0;
	while((i < sizeg) && (tmp[i] == bes[i]))
	    i++;
	if(i == sizeg)
	    break;

	// renewal
	for(i = 0; i < sizeg; i++)
	    bes[i] = tmp[i];
    }

    // bes *= (2n-1)(2n-3)...3 1
    for(i = 2; i <= n; i++)
	exflib_float_imul(bes, bes, 2*i-1, pr64g);

    // bes /= x^(n+1)
    term[0] = EXFLIB_BIAS;
    for(i = 1; i < sizeg; i++)
	term[i] = 0;
    for(i = n+1; i > 0; i >>= 1){
	if(i%2)
	    exflib_float_mul(term, term, xx, pr64g);
	exflib_float_mul(xx, xx, xx, pr64g);
    }
    exflib_float_div(bes, bes, term, pr64g);
    
    // return = -bes/(2^n)
    exflib::work.num[0] = bes[0] + EXFLIB_MASK_SGN;
    for(i = 1; i < exfloat::size; i++)
	exflib::work.num[i] = bes[i];

    return exflib::work;
}
#ifndef __BORLANDC__
//----------------------------------------------------------------------
//  support functions
//----------------------------------------------------------------------

//#include <new>
//#include <cstdlib>	// exit(1) library function
//#include <string>	// strerror
#include <errno.h>	// errno variable

namespace exflib {

    void memory_err()
    {
	std::cerr << "new: " << strerror(errno) << '\n';
	std::exit(1);
    }

    // memory allocation for 2 dimentional array (matrix)
    template <class _Tp>
    void newMatrix(_Tp **&matrix, int row, int col)
    {
	matrix = new _Tp* [row];
	matrix[0] = new _Tp [row * col];
	
	for(int i = 1; i < row; i++)
	    matrix[i] = matrix[0] + i*col;

	return;
    }

    template <class _Tp>
    void newMatrix(_Tp **&matrix, int size)
    {
	matrix = new _Tp* [size];
	matrix[0] = new _Tp [size * size];
	
	for(int i = 1; i < size; i++)
	    matrix[i] = matrix[0] + i*size;

	return;
    }

    template <class _Tp>
    void deleteMatrix(_Tp **a)
    {
	delete [] *a;
	delete [] a;
	return;
    }

    //  matrix multiplication
    template <class _Tp>
    void mulMatrix(const _Tp *const *const a, const _Tp *const *const b, int n,
		   _Tp *const *const c)
    {
	for(int i = 0; i < n; i++)
	    for(int j = 0; j < n; j++){
		c[i][j] = 0;
		for(int k = 0; k < n; k++)
		    c[i][j] += a[i][k]*b[k][j];
	    }
	return;
    }
};

//----------------------------------------------------------------------
//  eigen values & eigen vectors
//  a : (n x n) matrix, er : error level
//  l : (n x 1) vector : eigen value,
//  v : (n x n) matrix : n-th column holds an eigen vector of eigen value l[n]
//
// return value : 1 : converge in loop, 0 : not converge in loop
//----------------------------------------------------------------------

int jacobi(const exfloat *const *const a, int n, const exfloat& er, long loop,
	   exfloat *const l, exfloat *const *const v)
{
    static const exfloat PI = "#PI";

    int i, j, maxi = 0, maxj = 0, turn;
    exfloat amax, theta, foo;
    exfloat **r, **transr, **new_v, **new_m, **m;

    std::set_new_handler(exflib::memory_err);
    exflib::newMatrix(r, n);
    exflib::newMatrix(transr, n);
    exflib::newMatrix(new_v, n);
    exflib::newMatrix(new_m, n);
    exflib::newMatrix(m, n);

    for(i = 0; i < n; i++)
	for(j = 0; j < n; j++)
	    m[i][j] = a[i][j];

    turn = 0;
    // initialize v
    for(i = 0; i < n; i++)
	for(j = 0; j < n; j++)
	    v[i][j] = (i == j)? 1: 0;

    do{
	amax = 0;
	// find the maximum non-diagonal element of m
	for(i = 0; i < n; i++)
	    for(j = 0; j < n; j++)
		if((i != j) && (fabs(amax) < fabs(m[i][j]))){
		    maxi = i;
		    maxj = j;
		    amax = m[i][j];
		}

	// end condition
	if(fabs(amax) < er)
	    break;

	// find a rotation angle
	foo = 2* m[maxi][maxj]/(m[maxi][maxi] - m[maxj][maxj]);

	if(m[maxi][maxi] != m[maxj][maxj])
	    theta = atan(foo)/2;
	else
	    theta = PI/4;

	// make a rotation matrix
	for(i = 0; i < n; i++)
	    for(j = 0; j < n; j++){
		r[i][j] = (i == j)? 1: 0;
		transr[i][j] = (i == j)? 1: 0;
	    }

	r[maxi][maxi] = cos(theta);
	r[maxj][maxj] = cos(theta);
	r[maxj][maxi] = sin(theta);
	r[maxi][maxj] = -sin(theta);
	transr[maxi][maxi] = cos(theta);
	transr[maxj][maxj] = cos(theta);
	transr[maxi][maxj] = sin(theta);
	transr[maxj][maxi] = -sin(theta);

	// calculate new v and m
	exflib::mulMatrix(v, r, n, new_v);

	for(i = 0; i < n; i++)
	    for(j = 0; j < n; j++)
		v[i][j] = new_v[i][j];

	exflib::mulMatrix(transr, m, n, new_m);
	exflib::mulMatrix(new_m, r, n, m);

    } while(turn++ < loop);

    for(i = 0; i < n; i++)
	l[i] = m[i][i];

    exflib::deleteMatrix(r);
    exflib::deleteMatrix(transr);
    exflib::deleteMatrix(new_v);
    exflib::deleteMatrix(new_m);
    exflib::deleteMatrix(m);

    return (turn < loop)? 1: 0;
}

//----------------------------------------------------------------------
//  find a max eigen value & its eigen vector
//  m : (n x n) matrix, er : error level, l : max eigen value (not vector)
//  v : (n x 1) eigen vecotr of l
//----------------------------------------------------------------------

void maxEigenValue(const exfloat *const *const m, long n, const exfloat& er,
		   exfloat *const l, exfloat *const v)
{
    int i,j;
    exfloat pre_l;

    std::set_new_handler(exflib::memory_err);
    exfloat *x = new exfloat [n];
    
    // initialize of an eigen vector
    v[0] = 1;
    for(i = 1; i < n; i++)
        v[i] = 0;
    *l = 0;

    do{
        pre_l = *l;
        // calculate x
        for(i = 0; i < n; i++)
            x[i] = 0;
        for(i = 0; i < n; i++)
            for(j = 0; j < n; j++)
                x[i] += m[i][j] * v[j];
        // find a max value of x
        *l = x[0];
        for(i = 1; i < n; i++)
            if(fabs(*l) < fabs(x[i]))
                *l =  x[i];
        // calculate an eigen vector
        for(i = 0; i < n; i++)
            v[i] = x[i] / *l;

    }while(fabs(*l - pre_l) > er);

    delete [] x;

    return;
}

//----------------------------------------------------------------------
//  eigen values & eigen vectors
//  a : (n x n) matrix, er : error level,
//  l : (n x 1) vector : eigen value,
//  v : (n x n) matrix : n-th column holds an eigen vector of eigen value l[n]
//----------------------------------------------------------------------

void bekijyo(const exfloat *const *const a, long n, const exfloat& er,
	     exfloat *const l, exfloat *const *const v)
{
    int rank, i, j, order;
    exfloat *y, **m, **transa;
    exfloat norm, lambda;

    std::set_new_handler(exflib::memory_err);
    y = new exfloat [n];
    exflib::newMatrix(m, n);
    exflib::newMatrix(transa, n);

    for(int i = 0; i < n; i++)
	for(int j = 0; j < n; j++)
	    m[i][j] = a[i][j];
    
    rank = n;
    order = 0;
    while(rank-- > 0){

	maxEigenValue(m, n, er, l + order, v[order]);

	// transpose matrix a
	for(i = 0; i < n; i++)
	    for(j = 0; j < n; j++)
		transa[i][j] = m[j][i];
	
	maxEigenValue(transa, n, er, &lambda, y);
	norm = 0;
	for(i = 0; i < n; i++)
	    norm += v[order][i] * y[i];

	// new matrix
	for(i = 0; i < n; i++)
	    for(j = 0; j < n; j++)
		m[i][j] -= lambda * v[order][i] * y[j]/norm;

	order++;
    }

    // transpose eigen vectors
    for(i = 0; i < n; i++)
	for(j = 0; j < n; j++)
	    transa[i][j] = v[j][i];
    for(i = 0; i < n; i++)
	for(j = 0; j < n; j++)
	    v[i][j] = transa[i][j];

    exflib::deleteMatrix(transa);
    exflib::deleteMatrix(m);
    delete [] y;
    
    return;
}

//----------------------------------------------------------------------
//  Gauss-Jordan Elimination (by Numerical Recipes)
//----------------------------------------------------------------------

template <class Tp>
void gaussj(Tp **a, const long n, Tp *b)
{
    long *indxc, *indxr, *ipiv;
    long i, icol, irow, j, k, l, ll;
    Tp big, dum, pivinv, temp;

    indxc = new long [n];
    indxr = new long [n];
    ipiv = new long [n];

    for(j = 0; j < n; j++)
	ipiv[j] = 0;
    for(i = 0; i < n; i++){
	big = 0;
	for(j = 0; j < n; j++)
	    if(ipiv[j] !=  1)
		for(k = 0; k < n; k++){
		    if(ipiv[k] == 0){
			if(fabs(a[j][k]) >= big){
			    big = fabs(a[j][k]);
			    irow = j;
			    icol = k;
			}
		    } else if(ipiv[k] > 1){
			std::cerr << "gaussj: Singular Matrix-1\n";
			std::exit(1);
		    }
		}
	++(ipiv[icol]);
	if(irow != icol){
	    for(l = 0; l < n; l++){
		temp = a[irow][l];
		a[irow][l] = a[icol][l];
		a[icol][l] = a[irow][l];
	    }
	    temp = b[irow];
	    b[irow] = b[icol];
	    b[icol] = temp;
	}
	indxr[i] = irow;
	indxc[i] = icol;
	if(a[icol][icol] == 0){
	    std::cerr << "gaussj: Singular Matrix-2\n";
	    std::exit(1);
	}
	pivinv = 1/a[icol][icol];
	a[icol][icol] = 1;
	for(l = 0; l < n; l++) a[icol][l] *=  pivinv;
	b[icol] *=  pivinv;
	for(ll = 0; ll < n; ll++)
	    if(ll != icol){
		dum = a[ll][icol];
		a[ll][icol] = 0;
		for(l = 0; l < n; l++)
		    a[ll][l] -=  a[icol][l]*dum;
		b[ll] -=  b[icol]*dum;
	    }
    }
    for(l = n-1; l >= 0; l--){
	if(indxr[l] !=  indxc[l])
	    for(k = 0; k < n; k++){
		temp = a[k][indxr[l]];
		a[k][indxr[l]] = a[k][indxc[l]];
		a[k][indxc[l]] = temp;
	    }
    }
    delete [] ipiv;
    delete [] indxr;
    delete [] indxc;

    return;
}

template <class Tp>
void gaussj(Tp **a, const long n, Tp **b, const long m)
{
    long *indxc, *indxr, *ipiv;
    long i, icol, irow, j, k, l, ll;
    Tp big, dum, pivinv, temp;

    indxc = new long [n];
    indxr = new long [n];
    ipiv = new long [n];

    for(j = 0; j < n; j++)
	ipiv[j] = 0;
    for(i = 0; i < n; i++){
	big = 0;
	for(j = 0; j < n; j++)
	    if(ipiv[j] !=  1)
		for(k = 0; k < n; k++){
		    if(ipiv[k] == 0){
			if(fabs(a[j][k]) >= big){
			    big = fabs(a[j][k]);
			    irow = j;
			    icol = k;
			}
		    } else if(ipiv[k] > 1){
			std::cerr << "gaussj: Singular Matrix-1\n";
			std::exit(1);
		    }
		}
	++(ipiv[icol]);
	if(irow != icol){
	    for(l = 0; l < n; l++){
		temp = a[irow][l];
		a[irow][l] = a[icol][l];
		a[icol][l] = a[irow][l];
	    }
	    for(l = 0; l < m; l++){
		temp = b[irow][l];
		b[irow][l] = b[icol][l];
		b[icol][l] = temp;
	    }
	}
	indxr[i] = irow;
	indxc[i] = icol;
	if(a[icol][icol] == 0){
	    std::cerr << "gaussj: Singular Matrix-2\n";
	    std::exit(1);
	}
	pivinv = 1/a[icol][icol];
	a[icol][icol] = 1;
	for(l = 0; l < n; l++) a[icol][l] *=  pivinv;
	for(l = 0; l < m; l++) b[icol][l] *=  pivinv;
	for(ll = 0; ll < n; ll++)
	    if(ll != icol){
		dum = a[ll][icol];
		a[ll][icol] = 0.0;
		for(l = 0; l < n; l++)
		    a[ll][l] -=  a[icol][l]*dum;
		for(l = 0; l < m; l++)
		    b[ll][l] -=  b[icol][l]*dum;
	    }
    }
    for(l = n-1; l >= 0; l--){
	if(indxr[l] !=  indxc[l])
	    for(k = 0; k < n; k++){
		temp = a[k][indxr[l]];
		a[k][indxr[l]] = a[k][indxc[l]];
		a[k][indxc[l]] = temp;
	    }
    }
    delete [] ipiv;
    delete [] indxr;
    delete [] indxc;

    return;
}

//----------------------------------------------------------------------
//  LU Decomposition
//----------------------------------------------------------------------

namespace exflib {

    void luDecomp(exfloat ** a, long n)
    {
	long i, j, k;
	exfloat sum;

	for(j = 0; j < n; j++){			// LU decomposition
	    
	    for(i = 0; i <= j; i++){		// determin U
		sum = 0;
		for(k = 0; k <= i-1; k++)
		    sum += a[i][k] * a[k][j];
		a[i][j] -= sum;
	    }
	    for(i = j + 1; i < n; i++){		// determin L
		sum = 0;
		for(k = 0; k < j ; k++)
		    sum += a[i][k] * a[k][j];
		a[i][j] = (a[i][j] - sum)/ a[j][j];

	    }
	}
	return;
    }
};

//----------------------------------------------------------------------
// determinant of matrix
//  a : (n x n) matrix
//----------------------------------------------------------------------

exfloat detMatrix(exfloat ** a, long n)
{
    exflib::luDecomp(a, n);

    exfloat d = 1;

    for(int i = 0; i < n; i++)
	d *= a[i][i];

    return d;
}

//----------------------------------------------------------------------
// inverse matrix
//  a, m : (n x n) matrix
//----------------------------------------------------------------------

void invMatrix(exfloat ** a, long n, exfloat *const *const m)
{
    int i;
    
    exflib::luDecomp(a, n);

    exfloat *b = new exfloat [n];
    exfloat *y = new exfloat [n];
    exfloat sum;

    for(int r = 0; r < n; r++){

	for(i = 0; i < n; i++)
	    b[i] = 0;
	b[r] = 1;

	// back substitution
	y[0] = b[0];				// Ly = b
	for(i = 1; i < n; i++){
	    sum = 0;
	    for(int j = 0; j < i; j++)
		sum += a[i][j] * y[j];
	    y[i] = b[i] - sum;
	}

	b[n-1] = y[n-1]/ a[n-1][n-1];		// Ux = y
	for(i = n-2; i >=0; i--){
	    sum = 0;
	    for(int j = i + 1; j < n; j++)
		sum += a[i][j] * b[j];
	    b[i] = (y[i] - sum) / a[i][i];
	}

	// alignment
	for(i = 0; i < n; i++)
	    m[i][r] = b[i];
    }

    // free memory
    delete [] b;
    delete [] y;

    return;
}

//----------------------------------------------------------------------
//  norm of matrices
//   a : (n x n) matrix, d : dimension, er : error level
//----------------------------------------------------------------------

exfloat normMatrix(const exfloat *const *const a, long n, long d,
		   const exfloat er = "1e-6")
{
    exfloat norm = 0, tmp;
    int i, j;

    switch(d){

    case 1:	// 1 norm

	for(j = 0; j < n; j++){

	    tmp = 0;
	    for(i = 0; i < n; i++)
		tmp += fabs(a[i][j]);

	    if(norm < tmp)
		norm = tmp;
	}

        break;

    case 2:	// 2 norm
    {
	std::set_new_handler(exflib::memory_err);
	exfloat **ta, **m;
	exflib::newMatrix(ta, n);
	exflib::newMatrix(m, n);
	exfloat *v = new exfloat [n];

	for(i = 0; i < n; i++)
	    for(j = 0; j < n; j++)
		ta[i][j] = a[j][i];

	exflib::mulMatrix(a, ta, n, m);
	maxEigenValue(m, n, er, &norm, v);

	norm = sqrt(norm);

	exflib::deleteMatrix(ta);
	exflib::deleteMatrix(m);
	delete [] v;
    }
	break;

    default:	// infty norm

	for(i = 0; i < n; i++){

	    tmp = 0;
	    for(j = 0; j < n; j++)
		tmp += fabs(a[i][j]);

	    if(norm < tmp)
		norm = tmp;
	}
    }

    return norm;
}

//----------------------------------------------------------------------
// condition number
//  a : (n x n) matrix, d : dimension, er : error level
//----------------------------------------------------------------------

exfloat condMatrix(exfloat ** a, long n, long d,
		   const exfloat er = "1e-6")
{
    exfloat **m;
    exflib::newMatrix(m, n);
    exfloat condNumber;
    condNumber = normMatrix(a, n, d, er);
    invMatrix(a, n, m);
    condNumber *= normMatrix(m, n, d, er);
    exflib::deleteMatrix(m);

    return condNumber;
}

//----------------------------------------------------------------------
// solve a linear system 'Ax = b' by LU decomposition
// a : (n x n) matrix, b : (n x 1) vector, x : (n x 1) vector (solution)
//----------------------------------------------------------------------

void lu(exfloat ** a, long n, const exfloat *const b, exfloat *const x)
{
    long i, k;
    exfloat sum;
    exfloat *y;

    // memory allocation
    std::set_new_handler(exflib::memory_err);
    y = new exfloat [n];

    exflib::luDecomp(a, n);

    y[0] = b[0];			// Ly = b
    for(i = 1; i < n; i++){
	sum = 0;
	for(k = 0; k < i; k ++)
	    sum += a[i][k] * y[k];
	y[i] = b[i] - sum;
    }

    x[n-1] = y[n-1]/ a[n-1][n-1];		// Ux = y
    for(i = n-2; i >=0; i--){
	sum = 0;
	for(k = i + 1; k < n; k++)
	    sum += a[i][k] * x[k];
	x[i] = (y[i] - sum) / a[i][i];
    }

    // free memory
    delete [] y;

    return;
}

//----------------------------------------------------------------------
//  QR decomposition : a = qr
//  a : (n x m) matrix, q : (n x m) matrix, r : (m x m) matrix
//----------------------------------------------------------------------

void qr(const exfloat *const *const a, long row, long col,
	exfloat *const *const q, exfloat *const *const r)
{
    int i, j, k, l;
    exfloat s, ss, c, tmp;
    exfloat **ak, **qk, **qq, **qqq, *w, **an;

    // allocate memory
    exflib::newMatrix(ak, row, col);
    exflib::newMatrix(an, row, col);
    exflib::newMatrix(qk, row);
    exflib::newMatrix(qq, row);
    exflib::newMatrix(qqq, row);
    w = new exfloat [row];

    // copy matrix a
    for(i = 0; i < row; i++)
	for(j = 0; j < col; j++)
	    ak[i][j] = a[i][j];

    // initialize qqq
    for(i = 0; i < row; i++)
	for(j = 0; j < row; j++)
	    qqq[i][j] = 0;
    for(i = 0; i < row; i++)
	qqq[i][i] = 1;

    for(k = 0; k < col; k++){

	ss = 0;
	for(i = k; i < row; i++)
	    ss += ak[i][k]*ak[i][k];
	s = sqrt(ss);
	if(ak[k][k] < 0) s = -s; // prevent trancation errors

	// make w
	if(k) w[k-1] = 0;
	for(i = k; i < row; i++)
	    w[i] = ak[i][k];
	w[k] += s;

	c = 1/(ss + s*ak[k][k]);

	// make q_k
	for(i = 0; i < row; i++)
	    for(j = 0; j < row; j++)
		qk[i][j] = 0;
	for(i = k; i < row; i++)
	    for(j = k; j < row; j++)
		qk[i][j] = -c*w[i]*w[j];
	for(i = 0; i < row; i++)
	    qk[i][i] += 1;
	
	// renewal q
	for(i = 0; i < row; i++)
	    for(j = 0; j < row; j++)
		qq[i][j] = qqq[i][j];
	exflib::mulMatrix(qq, qk, row, qqq);

	// renewal a
	an[k][k] = -s;
	for(i = k+1; i < row; i++)
	    an[i][k] = 0;
	for(j = k+1; j < col; j++)
	    for(i = k; i < row; i++){
		tmp = 0;
		for(l = k; l < row; l++)
		    tmp += w[l]*ak[l][j];
		an[i][j] = ak[i][j] - c*w[i]*tmp;
	    }
	for(i = 0; i < row; i++)
	    for(j = 0; j < col; j++)
		ak[i][j] = an[i][j];
    }

    // alignment
    for(i = 0; i < row; i++)
	for(j = 0; j < col; j++)
	    q[i][j] = qqq[i][j];
    for(i = 0; i < col; i++)
	for(j = 0; j < col; j++)
	    r[i][j] = ak[i][j];

    // free memory
    exflib::deleteMatrix(ak);
    exflib::deleteMatrix(qk);
    exflib::deleteMatrix(qq);
    exflib::deleteMatrix(qqq);
    delete [] w;

    return;
}

#endif // __BORLANDC__

#endif // EXFUTIL_H

// End of exfutil.h
